# FILE: solver.tcl
#
# Class for controlling micromagnetic solver
#

Oc_Class mms_solver {
    private variable mif
    private variable grid
    private variable fieldrange
    public variable relax_stop_mxh = -1.
    ## Dummy value, set to real value when mif file is read in.
    public variable relax_stop_proc = "$this RelaxStopCheck"
    public variable stop_request_flag = 0

    private variable solver_state = Pause
    ## Should be one of Pause, OneStep, Relax or Run
    ## Set by calls to ChangeSolverState.  The 'after idle'
    ## method BackgroundStep checks the value of this variable at
    ## the start and end of processing.

    private variable BackgroundId = {}
    ## If not {}, then the 'after id' of the BackgroundStep script.
    ## If = {}, then no BackgroundStep is currently scheduled.

    private variable ode_step_count = 0  ;# Total number of ODE
    ## steps taken.  This is incremented until a reset or the loading
    ## of a new problem resets this back to 0.

    private variable simulation_time = 0; ;# Total simulation time in
    ## "simulation" seconds.  This is model time, not CPU process time.
    ## Like ode_step_count, this is only reset upon a reset or the
    ## loading of a new problem is.

    public variable relax_callback = Oc_Nop
    public variable fieldstep_callback = Oc_Nop

    # Events generated by this class:
    #     Iteration   --- ODE step complete
    #     Pause       --- Solver switch into Pause mode
    #     Equilibrium --- Equilibrium reached (at current field)
    #     FieldStep   --- Applied field nominal value stepped  
    #     FieldDone   --- FieldStep attempted, but no more steps in list
    #     delete      --- Instance being deleted


    private variable bindtags

    Constructor { _mif args } {
        if {[string match {} $_mif]} {
            error "Invalid mif object passed to mms_solver constructor"
        }
        mms_fieldrange New fieldrange
        set bindtags [list $this $class]
        eval $this Configure $args
        $this Reset $_mif
    }
    method Reset { {_mif {}} } {
        $this ChangeSolverState Pause
        if {![string match {} $_mif]} {
            if {[info exists mif]} {
                $mif Delete
		unset mif
            }
            if {[info exists grid]} {
                $grid Delete
                unset grid
            }
            if {![info exists fieldrange]} {
                mms_fieldrange New fieldrange  ;# Safety
            }
            mms_mif New mif $_mif  ;# Copy constructor
            $fieldrange SetFields [$mif GetFieldRangeList]
            set relax_stop_mxh [$mif GetConvergeMxHValue]
            mms_grid2D New grid $mif
        } else {
            $fieldrange Reset
            if {![info exists grid]} {
                error "Variable `grid' unset in mms_solver::Reset"
            }
            $grid Reset
        }
        $grid SetAppliedField \
		[$fieldrange GetField] [$fieldrange GetFieldIndex]
        set ode_step_count 0
        set simulation_time 0
    }
    method GetIterationValueLabels {} {
        return \
"  ODE     Max       Max                               ENERGIES
 Step     m x h     Angle     Total       Exchange    Anisotropy      Demag        Zeeman"
    }
    method GetMaxMxH {} {
        return [$grid GetMaxMxH]
    }
    method GetFieldUpdateCount {} {
        return [$grid GethUpdateCount]
    }
    method GetStepStats {} {
	return [$grid GetStepStats]
    }
    method GetIterationValues {} {
        set stepcount [$this GetODEStepCount]
        set mxh [$grid GetMaxMxH]
        set maxangle [$this GetMaxAngle]
        set energies [join [$this GetEnergyDensities]]
        return [format \
                "%5d %10.7f  %5.1f  %12.4f %12.4f %12.4f %12.4f %12.4f" \
                $stepcount $mxh $maxangle \
                [lindex $energies 1] [lindex $energies 3] \
                [lindex $energies 5] [lindex $energies 7] \
                [lindex $energies 9]]
    }
    method RelaxStopCheck { {mxh unknown} } {
        # Returns 1 if stopping criteria are met, 0 otherwise.
        # Call with mxh set to "unknown" (the default)
        # to query $grid for max m x h.
        if { [string match "unknown" $mxh] } {
            # Query grid for max m x h
            set mxh [$grid GetMaxMxH]
        }
        if {[expr $mxh<$relax_stop_mxh]} {
            return 1      ;# STOP!
        }
        return 0
    }

    ;### NOTE: The following method should be private, but is run via
    ;###   the update stack must be technically public.
    method BackgroundStep { {mxh_check 0} } {
        # This method should be called via 'after idle'.  At
        # the start of processing, this method first checks the
        # value of solver_state, and returns immediately if
        # solver_state==Pause.  Otherwise, one step is made
        # in the ODE, w/o any update calls.  After this, the
        # method may reschedule itself, depending on the value
        # of solver_state and RelaxStopCheck.

        # If mxh_check is true, then the code checks the
        # max_mxh value *instead* of taking an ODE step.
        # Rescheduling occurs the same either way, with
        # mxh_check set to 0 on the rescheduled event, so
        # a Relax or a Run sequence can be initiated by calling
        # this routine with mxh_check set to 1 to disable
        # unnecessary ODE stepping at the current field.
        if {[string match {} $BackgroundId]} {
            # Safety check
            error "solver method BackgroundStep call scheduling error"
        }

        set BackgroundId {}
        if {[string match Pause $solver_state]} { return }
        if {$mxh_check} {
            set mxh [$grid GetMaxMxH]
        } else {
            set mxh [$grid StepODE]
            incr ode_step_count
            set simulation_time [expr $simulation_time + [$grid GetTimeStep]]
            if {[catch {em_eventHandler Generate Iteration $bindtags} msg]} {
		bgerror $msg
	    }
        }

        # Check relax and field_step requirements, and generate
        # events and callbacks as appropriate.
        set relax_event 0
        set fieldstep_attempt 0
        if {[eval $relax_stop_proc $mxh]} {
            set relax_event 1
            if {[string match Run $solver_state]} {
		set fieldstep_attempt 1
            }
        }
        if {$relax_event} {
            # Relax events get called with old field values still
            # in place.
            eval $relax_callback
            if {[catch {em_eventHandler Generate Equilibrium $bindtags} \
		    msg]} {
		bgerror $msg
	    }
        }
	set fieldstep_event 0
        if {$fieldstep_attempt} {
            if {[$fieldrange GetFieldIndex]+1<[$fieldrange GetFieldCount]} {
                $this IncrementFieldStep
                eval $fieldstep_callback
		if {[catch {em_eventHandler Generate FieldStep $bindtags} \
			msg]} {
		    bgerror $msg
		}
		set fieldstep_event 1
            } else {
		if {[catch {em_eventHandler Generate FieldDone $bindtags} \
			msg]} {
		    bgerror $msg
		}
            }
        }

        # Reschedule if necessary, otherwise set solver_state to Pause.
	# (Also do safety check on $solver_state.)
	# NB: 'Pause' is a possible state, because the em_eventHandlers
	#     may modify the solver state.
        set reschedule 0
        switch -exact -- $solver_state {
            Run     {
                if {!$relax_event || $fieldstep_event} {
                    set reschedule 1
                }
            }
            Relax   {
                if {!$relax_event} {
                    set reschedule 1
                }
            }
	    Pause  -
            OneStep {}
            default {
                error "Invalid solver_state detected in\
                        BackgroundStep: ->${solver_state}<-"
            }
        }
        if {$reschedule} {
            set BackgroundId [after 0 \
                    set [$this GlobalName BackgroundId] \
                    \[after idle [list $this BackgroundStep 0]\]]
            ## We need to schedule this from the non-idletask stack,
            ## because if Tcl_DoOneEvent processes *any* idletasks, it
            ## processes *all*, including apparently ones that go into
            ## the list after idletask processing has commenced.  The
            ## above seems to work, but Dean Brettle <brettle@his.com>
            ## claims that there is a race condition in this as well
            ## (see his comp.lang.tcl posting on 1998/02/15).  I haven't
            ## looked at the Tcl core source code, but it may be that
            ## a Tcl 'update' call with the above in place may be evil
            ## as well.  If a problem is observed with the above, it is
            ## likely that changing the 'after 0' to, say, 'after 20' will
            ## better hide the problem.  -mjd, 1998/04/07.
        } else {
            set solver_state "Pause"
	    if {[catch {em_eventHandler Generate Pause $bindtags} msg]} {
		bgerror $msg
	    }
        }
    }

    method ChangeSolverState { newstate } {
        # 'newstate' should be one of OneStep, Relax, Run or Pause.
        if {[string match $solver_state $newstate]} {return} ;# No change

        # Rather than relying on the state of BackgroundID, play
        # it safe and always de-schedule BackgroundId, and
        # re-schedule if necessary.  (Presumably ChangeSolverState
        # won't be called inside a tight loop!)
        if {![string match {} BackgroundId]} {
            after cancel $BackgroundId
            set BackgroundId {}
        }

        switch -exact -- $newstate {
            OneStep {
                set BackgroundId [after idle [list $this BackgroundStep 0]]
            }
            Relax   -
            Run    {
                set BackgroundId [after idle [list $this BackgroundStep 1]]
            }
            Pause {}
            default {
                error "Invalid solver_state request to\
                        ChangeSolverState: $newstate"
            }
        }
        set solver_state $newstate
        if {[string match Pause $solver_state]} {
	    if {[catch {em_eventHandler Generate Pause $bindtags} msg]} {
		bgerror $msg
	    }
        }
    }
    method RelaxState { {new_relax_callback Oc_Nop} } {
        # Schedules solver for background processing and
        # returns.
;### error "mms_solver method RelaxState is obsolete"
        set relax_callback $new_relax_callback
        $this ChangeSolverState Relax
        while {[string compare Relax $solver_state]==0} {
            vwait [$this GlobalName solver_state]
        }
    }
    method HystLoop { {new_relax_callback Oc_Nop} \
            { new_fieldstep_callback Oc_Nop } } {
        # Schedules solver for background processing and
        # returns.
;### error "mms_solver method HystLoop is obsolete"
        set relax_callback $new_relax_callback
        set fieldstep_callback $new_fieldstep_callback
        $this ChangeSolverState Run
        while {[string compare Run $solver_state]==0} {
            vwait [$this GlobalName solver_state]
        }
    }
    method IncrementFieldStep { { amount 1 }} {
        set newindex [$fieldrange StepField $amount]
        $grid SetAppliedField \
		[$fieldrange GetField] [$fieldrange GetFieldIndex]
        return $newindex
    }
    method GetFieldStepCount {} {
        return [$fieldrange GetFieldIndex]
    }
    method GetODEStepCount {} { return $ode_step_count; }
    method GetMaxAngle {} {
        # Returns max neighbor angle in degrees.
        return [expr [$grid GetMaxNeighborAngle]*180./3.14159265358979]
    }
    method GetEnergyDensities {} {
        # Returns a list of list pairs, energy name + value
        # The first is the total energy"
        set energy_list [$grid GetEnergyDensities]
        set rtn_list {}
        lappend rtn_list [list "Total" [lindex $energy_list 4]]
        lappend rtn_list [list "Exchange" [lindex $energy_list 0]]
        lappend rtn_list [list "Anisotropy" [lindex $energy_list 1]]
        lappend rtn_list [list "Demag" [lindex $energy_list 2]]
        lappend rtn_list [list "Zeeman" [lindex $energy_list 3]]
        return $rtn_list
    }
    method GetNomAppliedField { {mult 1.} {fmt "%.10g %.10g %.10g"}} {
        set field [$fieldrange GetField]
        return [format $fmt \
                [expr [lindex $field 0]*$mult] \
                [expr [lindex $field 1]*$mult] \
                [expr [lindex $field 2]*$mult]]
    }
    method GuessAppliedFieldAxis {} {
        #  Queries the mif file and tries to guess the component
        # of BApplied that is most likely intended for M-B loop.
        #  Returns a unit 3 vector.
        # NB: This method is a serious hack.  Its use is discouraged.
        set B0 [$fieldrange GetField 0]
        set B1 [$fieldrange GetField 1]
        set dBx [expr abs([lindex $B1 0] - [lindex $B0 0])]
        set dBy [expr abs([lindex $B1 1] - [lindex $B0 1])]
        set dBz [expr abs([lindex $B1 2] - [lindex $B0 2])]
        if { $dBz > $dBy && $dBz > $dBx } { return "0 0 1" }
        if { $dBy > $dBx }                { return "0 1 0" }
        return "1 0 0"
    }
    method GetAveMag {} {
        # Returns average magnetization M/Ms, as a string with
        # 3 entries.
        set mag [$grid GetAveMag]
        return "[lindex $mag 0] [lindex $mag 1] \
                [lindex $mag 2]"
    }
    method GetStepSize {} {
        return [$grid GetStepSize]
    }
    method GetTimeStep {} {
        return [$grid GetTimeStep]
    }
    method GetSimulationTime {} {
        return $simulation_time
    }
    method GetOutputBasename {} {
        return [$mif GetOutBaseName]
    }
    method GetSolverState {} {
        return $solver_state
    }
    method WriteFieldFile { {filename {}} {note " "} } {
        # Writes reduced h field to a ovf file
        if {[string match {} $filename]} {
            # Use data in mif to construct "real" filename
            set filename [format "%s.pass%05d.ohf" \
                    [$mif GetOutBaseName] [$this GetODEStepCount]]
            set filename [Oc_DirectPathname $filename] 
            # Get arb. temp name
            Oc_TempFile New t -stem solveh -extension .ohf
            set tempname [$t AbsoluteName]
            $t Delete
        } else {
            set tempname [Oc_DirectPathname $filename]
            set filename {}
        }

        # If $note is a single blank (default), then fill in Happ and
        # iteration information.
        if {[string match " " $note]} {
            set note "Applied field (T): [$this GetNomAppliedField]\
                    \nIteration: [$this GetODEStepCount]\
                    \n|m x h|: [$grid GetMaxMxH]"
        }

        # Write file
        $grid WriteOvf h binary4 $tempname $filename $note
        return [list $tempname $filename]
    }
    method WriteMagFile { {filename {}} {note " "} } {
        # Writes magnetization to a ovf file
        if {[string match {} $filename]} {
            # Use data in mif to construct "real" filename
            set filename [format "%s.pass%05d.omf" \
                    [$mif GetOutBaseName] [$this GetODEStepCount]]
            set filename [Oc_DirectPathname $filename] 
            # Get arb. temp name
            Oc_TempFile New t -stem solvem -extension .omf
            set tempname [$t AbsoluteName]
            $t Delete
        } else {
            set tempname [Oc_DirectPathname $filename]
            set filename {}
        }

        # If $note is a single blank (default), then fill in Happ and
        # iteration information
        if {[string match " " $note]} {
            set note "Applied field (T): [$this GetNomAppliedField]\
                    \nIteration: [$this GetODEStepCount]\
                    \n|m x h|: [$grid GetMaxMxH]"
        }

        # Write file
        $grid WriteOvf m binary4 $tempname $filename $note
        return [list $tempname $filename]
    }
    Destructor {
        if {[info exists bindtags] } {
            catch { em_eventHandler Generate delete $bindtags }
        }
        if {[info exists grid]} {
            catch { $grid Delete }
        }
        if {[info exists mif]} {
            catch { $mif Delete }
        }
        if {[info exists fieldrange]} {
            catch { $fieldrange Delete }
        }
    }
}
