/* FILE: errhandlers.h                    -*-Mode: c++-*-
 *
 * Some error handling routines that should be phased out
 * and replaced with code in the oc extension.
 * 
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 1998/08/07 23:23:11 $
 * Last modified by: $Author: donahue $
 */

#ifndef _NB_ERRHANDLERS
#define _NB_ERRHANDLERS

#include "dstring.h"

/* End includes */     /* Optional directive to build.tcl */

//////////////////////////////////////////////////////////////////////////
// Class to store error & warning messages in a global Nb_DString
class MessageLocker
{
private:
  static Nb_DString msg;
public:
  static int Reset() { msg.Free(); return 0; }
  static int Append(const CHAR *fmt,...);
  static int GetMessage( Nb_DString& _msg);   // Dup's msg into _msg.
};

// Routine to print message to standard (user) output channel
int Message(const CHAR *message_fmt,...);

// C-interface to Tcl/Tk error boxes.  The above routines map
// into this.  This routine is defined in tkcmds.cc
enum MessageType { MSG_DEBUG, MSG_INFO, MSG_WARNING, MSG_ERROR };
INT4m TkMessage(MessageType mt,const CHAR *msg,INT4m exitcode=1);

//////////////////////////////////////////////////////////////////////////
// Error codes
extern const char* ErrArrOutOfBounds;
extern const char* ErrBadFile;
extern const char* ErrBadFileHeader;
extern const char* ErrBadFileOpen;
extern const char* ErrInputEOF;
extern const char* ErrOutputEOF;
extern const char* ErrBadParam;
extern const char* ErrDispInit;
extern const char* ErrMisc;
extern const char* ErrNoMem;
extern const char* ErrOverflow;
extern const char* ErrProgramming;

//////////////////////////////////////////////////////////////////////////
// Error handlers
// Note: Use preprocessor macros __FILE__ and __LINE__ to get
//       'filename' and 'lineno' respectively.
void FatalError(INT4m errorcode,
		const CHAR *membername,
		const ClassDoc &class_doc,
		const CHAR *filename,INT4m lineno,
		const CHAR *fmt,...);
/// Doesn't return; exits with code 'errorcode'

void NonFatalError(const CHAR *membername,
		   const ClassDoc &class_doc,
		   const CHAR *filename,INT4m lineno,
		   const CHAR *fmt,...);
/// Prints warning message, blocks if Tk is enabled, and returns.

void ClassMessage(const CHAR *membername,
		  const ClassDoc &class_doc,
		  const CHAR *filename,INT4m lineno,
		  const CHAR *fmt,...);
/// Prints warning message and returns. Non-blocking

void ClassDebugMessage(const CHAR *membername,
		       const ClassDoc &class_doc,
		       const CHAR *filename,INT4m lineno,
		       const CHAR *fmt,...);
/// Prints warning message and returns. Non-blocking

// Versions of the above for non-class functions:
void PlainWarning(const CHAR *fmt,...);
void PlainError(INT4m errorcode,const CHAR *fmt,...);

// If you #define MEMBERNAME at the start of each member definition,
// then you can use the following macro to expand to 4 of the fields
// needed by the above error handlers.
#define STDDOC MEMBERNAME,class_doc,__FILE__,__LINE__

#endif // _NB_ERRHANDLERS
