# FILE: filedlg.tcl
#
# Generic file dialog box

Oc_Class Ow_FileDlg {
    const public variable winpath
    const public variable outer_frame_options = "-bd 2 -relief raised"
    const public variable filter = "*"
    const public variable dialog_title = "File Select"
    const public variable selection_title = "Selection"
    const public variable select_action_id = "SELECT"
    const public variable optionbox_setup = {} ;# Optional option widget
    ## setup callback.  If set != {}, Ow_FileDlg will call this routine
    ## during initialization with the name this Ow_FileDlg widget, and
    ## the name of a frame widget.  The option setup routine should then
    ## construct itself inside that frame widget. Currently, no
    ## additional callbacks to the option widget are made. It is the
    ## responsibility of the Ow_FileDlg caller to query the option
    ## widget as necessary on general callbacks from Ow_FileDlg.
    public variable callback = Oc_Nop
    public variable delete_callback_arg = {}
    private variable filewidget = {}
    private variable selectionwidget = {}
    private variable dirwidget = {}
    Constructor {args} {
	eval $this Configure -winpath .w$this $args
	# Create toplevel frame to hold dialog
	toplevel $winpath
	wm group $winpath .
	wm title $winpath $dialog_title
	if {![string match {} [info commands Ow_PositionChild]]} {
	    Ow_PositionChild $winpath  ;# Position at +.25+.25 over '.'
	}
	wm protocol $winpath WM_DELETE_WINDOW \
		"$this SwitchBoard $this CANCEL"

	# Temporarily disable callbacks until construction is complete
	set callback_keep $callback ;	set callback Oc_Nop

	# Create subwidgets
	Ow_FileSelect New filewidget $winpath \
		-callback "$this SwitchBoard" \
		-filter $filter
	set neframe [frame $winpath.neframe] ;# Packing frame
	if {![string match {} $optionbox_setup]} {
	    # Optional options widget requested
	    set optframe [frame $winpath.optframe -bd 4 -relief ridge]
	    $optionbox_setup $this $optframe
	} else {
	    set optframe {}
	}
	set ctrlframe [frame $winpath.ctrlframe]
        button $winpath.ctrlframe.rescan -text "Rescan" \
                -command [list $this SwitchBoard $this RESCAN]
        button $winpath.ctrlframe.cancel -text "Cancel" \
                -command [list $this SwitchBoard $this CANCEL]
        button $winpath.ctrlframe.ok -text "OK" \
                -command [list $this SwitchBoard $this SELECT]
	Ow_SelectBox New selectionwidget $winpath \
		-title $selection_title \
		-callback "$this SwitchBoard"
	Ow_DirSelect New dirwidget $winpath \
		-callback "$this SwitchBoard"

	# Initialize selection path
	$selectionwidget SetPath [$dirwidget Cget -value]

	# Pack subwidgets
	pack [$filewidget Cget -winpath] -side left \
		-fill both -expand 1 -padx 2m -pady 2m
	pack $winpath.ctrlframe.ok -in $ctrlframe \
		-side right -expand 0 -anchor ne -padx 2m -pady 2m
	pack $winpath.ctrlframe.cancel -in $ctrlframe \
		-side right -expand 0 -anchor ne -padx 2m -pady 2m
	pack $winpath.ctrlframe.rescan -in $ctrlframe \
		-side right -expand 0 -anchor ne -padx 2m -pady 2m
	pack $ctrlframe -side right -in $neframe \
		-expand 0 -anchor ne -padx 2m -pady 2m
	if {![string match {} $optframe]} {
	    pack $optframe -side left -in $neframe \
		    -expand 0 -anchor nw -padx 2m -pady 2m
	}
	pack $neframe -side top \
		-fill x -expand 0 -anchor ne -padx 2m -pady 2m
	pack [$selectionwidget Cget -winpath] -side top \
		-fill x -expand 0 -anchor n -padx 2m -pady 2m
	pack [$dirwidget Cget -winpath] -side right \
		-fill both -expand 1 -padx 2m -pady 2m

	# Widget bindings
	bind $winpath <Destroy> "+$this WinDestroy %W"

	# Re-enable callbacks
	set callback $callback_keep
    }
    method GetFilename {} { selectionwidget } {
	# Returns full path+filename of selected value
	return [$selectionwidget ReadDisplayedFullname]
    }
    method SwitchBoard { subwidget actionid args } \
	    { filewidget selectionwidget dirwidget \
	      callback select_action_id } {
	# General callback routing routine
	switch -exact -- $actionid {
	    UPDATE {
		# Update actions
		if {[string compare $subwidget $filewidget]==0} {
			$selectionwidget SetFilename [lindex $args 1]
		} elseif {[string compare $subwidget $dirwidget]==0} {
			$filewidget SetPath [lindex $args 1]
			$selectionwidget SetPath [lindex $args 1]
		} else {
		    error "Unknown UPDATE callback source: [\
			    $subwidget class]"
		}
	    }
	    SELECT {
		set errmsg [eval $callback $this $select_action_id]
		if { [string match {} $errmsg] } {
		    $this Delete ; return
		}
		$filewidget Rescan
		$dirwidget Rescan
	    }
	    RESCAN {
		$filewidget Rescan
		$dirwidget Rescan
	    }
	    CANCEL {
		$this Delete ; return
	    }
	    DELETE {}
	    default { error "Unknown actionid: $actionid" }
	}
    }
    method WinDestroy { w } { winpath } {
	if {[info exists winpath] && [string compare $winpath $w]!=0} {
	    return    ;# Child destroy event
	}
	$this Delete
    }
    private variable delete_in_progress = 0
    Destructor {
	# Protect against re-entrancy
	if { [info exists delete_in_progress] && \
		$delete_in_progress } { return }
	set delete_in_progress 1

	# Notify client of impending destruction
	catch {eval $callback $this DELETE {$delete_callback_arg}}

	# Delete subwidgets
	catch {$filewidget      Delete}
	catch {$selectionwidget Delete}
	catch {$dirwidget       Delete}

	# Delete window
	if {[info exists winpath] && [winfo exists $winpath]} {
	    # Destroy instance frame, all children, and bindings
	    destroy $winpath
	}
    }
}
